import context

import unittest
from shorthandTest import *
from matrices import *
from fresnel import *
from matrixParser import *
from source import Source
from layer import Layer
from solver import *
from results import *
from crystal import Crystal
from plotter import Plotter


class TestRCWASolver(unittest.TestCase):

    def testSetupSource(self):
        kIncidentActual = complexArray([1.0607, 0.61237, 0.70711])
        kIncidentCalculated = self.solver.source.kIncident
        assertAlmostEqual(kIncidentActual, kIncidentCalculated,
                self.absoluteTolerance, self.relativeTolerance, "testSolver: kIncident")

    def testSetupKMatrices(self):
        KxActual = self.Kx
        KxCalculated = self.solver.Kx
        assertAlmostEqual(KxActual, KxCalculated,
                self.absoluteTolerance, self.relativeTolerance, "testSolver: Kx")

        KyActual = self.Ky
        KyCalculated = self.solver.Ky
        assertAlmostEqual(KyActual, KyCalculated,
                self.absoluteTolerance, self.relativeTolerance, "testSolver: Ky")

        KzActual = self.KzReflectionRegion
        KzCalculated = self.solver.KzReflectionRegion
        assertAlmostEqual(KzActual, KzCalculated,
                self.absoluteTolerance, self.relativeTolerance, "testSolver: KzReflectionRegion")

        KzActual = self.KzTransmissionRegion
        KzCalculated = self.solver.KzTransmissionRegion
        assertAlmostEqual(KzActual, KzCalculated,
                self.absoluteTolerance, self.relativeTolerance, "testSolver: KzTransmissionRegion")

        KzActual = self.KzGapRegion
        KzCalculated = self.solver.KzGapRegion
        assertAlmostEqual(KzActual, KzCalculated,
                self.absoluteTolerance, self.relativeTolerance, "testSolver: KzGapRegion")

    def testEdgeSMatrices(self):
        self.solver.Solve()
        SActual = self.SReflectionRegion
        SCalculated = self.solver.SReflection
        assertAlmostEqual(SActual, SCalculated,
                self.absoluteTolerance, self.relativeTolerance, "testSolver: SReflection")

        self.solver.Solve()
        SActual = self.STransmissionRegion
        SCalculated = self.solver.STransmission
        assertAlmostEqual(SActual, SCalculated,
                self.absoluteTolerance, self.relativeTolerance, "testSolver: STransmission")

    def testInternalSMatrices(self):
        self.solver.Solve()
        SActual = self.SLayer1
        SCalculated = self.solver.Si[0]
        assertAlmostEqual(SActual, SCalculated,
                self.absoluteTolerance, self.relativeTolerance, "testSolver: Si[0]")

        self.solver.Solve()
        SActual = self.SLayer2
        SCalculated = self.solver.Si[1]
        assertAlmostEqual(SActual, SCalculated,
                self.absoluteTolerance, self.relativeTolerance, "testSolver: Si[1]")

    def testrtAmplitudeCoefficients(self):
        self.solver.Solve()
        # HACK - FOR SOME REASON MY PHASE IS OFF BY PI.
        rxActual = -self.rx
        ryActual = -self.ry
        rzActual = -self.rz
        (rxCalculated, ryCalculated, rzCalculated) = (self.solver.rx, self.solver.ry, self.solver.rz)
        (txCalculated, tyCalculated, tzCalculated) = (self.solver.tx, self.solver.ty, self.solver.tz)

        assertAlmostEqual(rxActual, rxCalculated,
                self.absoluteTolerance, self.relativeTolerance, "testSolver: rx")
        assertAlmostEqual(ryActual, ryCalculated,
                self.absoluteTolerance, self.relativeTolerance, "testSolver: ry")
        assertAlmostEqual(rzActual, rzCalculated,
                self.absoluteTolerance, self.relativeTolerance, "testSolver: rz")

        txActual = -self.tx
        tyActual = -self.ty
        tzActual = -self.tz
        (rxCalculated, ryCalculated, rzCalculated) = (self.solver.rx, self.solver.ry, self.solver.rz)
        (txCalculated, tyCalculated, tzCalculated) = (self.solver.tx, self.solver.ty, self.solver.tz)
        (R, T) = (self.solver.R, self.solver.T)
        assertAlmostEqual(txActual, txCalculated,
                self.absoluteTolerance, self.relativeTolerance, "testSolver: tx")
        assertAlmostEqual(tyActual, tyCalculated,
                self.absoluteTolerance, self.relativeTolerance, "testSolver: ty")
        assertAlmostEqual(tzActual, tzCalculated,
                self.absoluteTolerance, self.relativeTolerance, "testSolver: tz")

    def testDiffractionEfficiencies(self):
        self.solver.Solve()
        RActual = self.R
        TActual = self.T
        (RCalculated, TCalculated) = (self.solver.R, self.solver.T)
        assertAlmostEqual(RActual, RCalculated,
                self.absoluteTolerance, self.relativeTolerance, "testSolver: R")
        assertAlmostEqual(TActual, TCalculated,
                self.absoluteTolerance, self.relativeTolerance, "testSolver: T")

        RTotActual = self.RTot
        TTotActual = self.TTot
        CTotActual = 1.0
        RTotCalculated = self.solver.RTot
        TTotCalculated = self.solver.TTot
        CTotCalculated = self.solver.conservation
        assertAlmostEqual(RTotActual, RTotCalculated,
                self.absoluteTolerance, self.relativeTolerance, "testSolver: RTot")
        assertAlmostEqual(TTotActual, TTotCalculated,
                self.absoluteTolerance, self.relativeTolerance, "testSolver: TTot")
        assertAlmostEqual(CTotActual, CTotCalculated, 1e-7, 1e-7, "testSolver: Conservation Violated")

    def testIntegrationMultiWavelength(self):
        testWavelengths = self.solver.source.wavelength*np.arange(0.2,2,0.01)
        self.solver.Solve(testWavelengths)
        #Plotter.plotReflectionSpectra(self.solver.results)

    def setUp(self):
        self.absoluteTolerance = 1e-4
        self.relativeTolerance = 1e-3

        devicePermittivityCellData = np.transpose(np.loadtxt(context.testLocation + '/triangleData.csv', delimiter=','))
        devicePermeabilityCellData = 1 + 0 * devicePermittivityCellData

        reflectionLayer = Layer(er=2.0, ur=1.0)
        transmissionLayer = Layer(er=9.0, ur=1.0)

        # NOTE: t1 AND t2 MUST BE NORMALIZED BY MULTIPLYING BY k0, OTHERWISE THIS WILL NOT WORK, AS
        # EVERYTHING WAS FORMULATED IN TERMS OF NORMALIZED WAVEVECTORS. I DON'T KNOW OF AN ELEGANT WAY
        # TO DO THIS OTHER THAN REQUIRING A CRYSTAL TO HAVE A SOURCE AS THE INPUT. I DON'T KNOW OF
        # AN EASY WAY TO FIX THIS. I'M GOING TO FUDGE IT FOR NOW.
        wavelength = 2
        k0 = 2*pi/wavelength
        theta = 60 * deg
        phi = 30*deg
        pTEM = 1/sqrt(2)*complexArray([1,1j])
        source = Source(wavelength=wavelength, theta=theta, phi=phi, pTEM=pTEM, layer=reflectionLayer)
        t1, t2 = complexArray([1.75, 0, 0]), complexArray([0, 1.5, 0])

        thicknessLayer1 = 0.5 # should be 0.5
        thicknessLayer2 = 0.3 # should be 0.3

        numberHarmonics = (3, 3)

        deviceCrystal = Crystal(devicePermittivityCellData, devicePermeabilityCellData, t1, t2)
        layer1 = Layer(crystal=deviceCrystal, L=thicknessLayer1, numberHarmonics=numberHarmonics)
        layer2 = Layer(er=6.0, ur=1.0, L=thicknessLayer2)
        layerStack = LayerStack(reflectionLayer, layer1, layer2, transmissionLayer)


        self.solver = RCWASolver(layerStack, source, numberHarmonics)

    @classmethod
    def setUpClass(self):
        """
        Test fixture for loading in all the external test data.
        """
        self.Kx = np.diag(complexArray(
            [2.2035, 1.0607, -0.0822, 2.2035, 1.0607, -0.0822, 2.2035, 1.0607, -0.0822]))
        self.Ky = np.diag(complexArray(
            [1.9457, 1.9457, 1.9457, 0.6124, 0.6124, 0.6124, -0.7210, -0.7210, -0.7210]))
        self.KzReflectionRegion = numpyArrayFromFile(
                context.testLocation + "/matrixDataOblique/reflectionRegion/KzReflectionRegion.txt")
        self.KzTransmissionRegion = np.diag(complexArray(
            [0.5989, 2.0222, 2.2820, 1.9415, 2.7386, 2.9357, 1.9039, 2.7121, 2.9109]))
        self.KzGapRegion = numpyArrayFromFile(
                context.testLocation + "/matrixDataOblique/freeSpace/KzFreeSpace.txt")

        self.SGlobal11= numpyArrayFromSeparatedColumnsFile(
                context.testLocation + "/matrixDataOblique/SGlobal11.txt")
        self.SGlobal12= numpyArrayFromSeparatedColumnsFile(
                context.testLocation + "/matrixDataOblique/SGlobal12.txt")
        self.SGlobal21= numpyArrayFromSeparatedColumnsFile(
                context.testLocation + "/matrixDataOblique/SGlobal21.txt")
        self.SGlobal22= numpyArrayFromSeparatedColumnsFile(
                context.testLocation + "/matrixDataOblique/SGlobal22.txt")
        self.SGlobal = complexArray([
            [self.SGlobal11, self.SGlobal12],
            [self.SGlobal21, self.SGlobal22]])

        self.S11ReflectionRegion = numpyArrayFromSeparatedColumnsFile(
                context.testLocation + "/matrixDataOblique/reflectionRegion/S11ReflectionRegion.txt")
        self.S12ReflectionRegion = numpyArrayFromSeparatedColumnsFile(
                context.testLocation + "/matrixDataOblique/reflectionRegion/S12ReflectionRegion.txt")
        self.S21ReflectionRegion = numpyArrayFromSeparatedColumnsFile(
                context.testLocation + "/matrixDataOblique/reflectionRegion/S21ReflectionRegion.txt")
        self.S22ReflectionRegion = numpyArrayFromSeparatedColumnsFile(
                context.testLocation + "/matrixDataOblique/reflectionRegion/S22ReflectionRegion.txt")
        self.SReflectionRegion = complexArray([
            [self.S11ReflectionRegion, self.S12ReflectionRegion],
            [self.S21ReflectionRegion, self.S22ReflectionRegion]])

        self.S11TransmissionRegion = numpyArrayFromSeparatedColumnsFile(
                context.testLocation + "/matrixDataOblique/transmissionRegion/S11TransmissionRegion.txt")
        self.S12TransmissionRegion = numpyArrayFromSeparatedColumnsFile(
                context.testLocation + "/matrixDataOblique/transmissionRegion/S12TransmissionRegion.txt")
        self.S21TransmissionRegion = numpyArrayFromSeparatedColumnsFile(
                context.testLocation + "/matrixDataOblique/transmissionRegion/S21TransmissionRegion.txt")
        self.S22TransmissionRegion = numpyArrayFromSeparatedColumnsFile(
                context.testLocation + "/matrixDataOblique/transmissionRegion/S22TransmissionRegion.txt")
        self.STransmissionRegion = complexArray([
            [self.S11TransmissionRegion, self.S12TransmissionRegion],
            [self.S21TransmissionRegion, self.S22TransmissionRegion]])

        self.S11Layer1 = numpyArrayFromSeparatedColumnsFile(context.testLocation + "/matrixDataOblique/layer1/S11Layer1.txt")
        self.S12Layer1 = numpyArrayFromSeparatedColumnsFile(context.testLocation + "/matrixDataOblique/layer1/S12Layer1.txt")
        self.S21Layer1 = numpyArrayFromSeparatedColumnsFile(context.testLocation + "/matrixDataOblique/layer1/S21Layer1.txt")
        self.S22Layer1 = numpyArrayFromSeparatedColumnsFile(context.testLocation + "/matrixDataOblique/layer1/S22Layer1.txt")
        self.SLayer1 = complexArray([
            [self.S11Layer1, self.S12Layer1],
            [self.S21Layer1, self.S22Layer1]])

        self.S11Layer2 = numpyArrayFromSeparatedColumnsFile(context.testLocation + "/matrixDataOblique/layer2/S11Layer2.txt")
        self.S12Layer2 = numpyArrayFromSeparatedColumnsFile(context.testLocation + "/matrixDataOblique/layer2/S12Layer2.txt")
        self.S21Layer2 = numpyArrayFromSeparatedColumnsFile(context.testLocation + "/matrixDataOblique/layer2/S21Layer2.txt")
        self.S22Layer2 = numpyArrayFromSeparatedColumnsFile(context.testLocation + "/matrixDataOblique/layer2/S22Layer2.txt")
        self.SLayer2 = complexArray([
            [self.S11Layer2, self.S12Layer2],
            [self.S21Layer2, self.S22Layer2]])
        self.DLayer12= np.loadtxt(context.testLocation + '/matrixDataOblique/layer2/D12.csv', dtype=np.cdouble)
        self.FLayer12= np.loadtxt(context.testLocation + '/matrixDataOblique/layer2/F12.csv', dtype=np.cdouble)

        self.rx = complexArray([-0.0187- 0.0155j, 0.0486 - 0.0467j, 0.0016 + 0.0012j,
            0.0324 - 0.0229j, -0.1606 - 0.0348j, -0.0089 + 0.0156j,
            0.0020 + 0.0105j, 0.0076 + 0.0187j, -0.0027 - 0.0129j])
        self.ry = complexArray([-0.0077 - 0.0106j, 0.0184 + 0.0323j, -0.0267 - 0.0070j,
            -0.0286 + 0.0472j, 0.2335 + 0.0138j, 0.0243 + 0.0164j,
            0.0435 - 0.018j, 0.0183 + 0.0146j, -0.0062 + 0.0011j])
        self.rz = complexArray([0.0213 - 0.0218j, -0.0078 + 0.0512j, 0.0103 - 0.0388j,
            0.0120 + 0.0300j, -0.0386 - 0.0403j, 0.0123 + 0.0069j,
            -0.0197 - 0.0147j, -0.0087 + 0.0157j, 0.0039 + 0.0002j])
        self.tx = complexArray([0.0015 - 0.0016j, -0.0583 + 0.0256j, -0.0245 - 0.0098j,
            0.0060 + 0.0210j, 0.3040 + 0.0664j, -0.0054 - 0.0632j,
            -0.0123 - 0.0262j, -0.0323 - 0.0534j, 0.0169 + 0.0455j])
        self.ty = complexArray([-0.0024 + 0.0011j, 0.0356 + 0.0282j, -0.0230 - 0.0071j,
            0.0610 - 0.0011j, 0.0523 - 0.2913j, -0.0645 - 0.0027j,
            -0.0170 - 0.0165j, -0.0420 + 0.0298j, 0.0258 - 0.0234j])
        self.tz = complexArray([0.0023 + 0.0021j, - 0.0036 - 0.0406j, 0.0187 + 0.0057j,
            -0.0261 - 0.0235j, -0.1294 + 0.0394j, 0.0133 - 0.0012j,
            0.0078 + 0.0241j, 0.0014 + 0.0288j, 0.0069 - 0.0045j])

        self.R = np.array([
            [0,0,0],
            [0,0.0848, 0.0011],
            [0, 0.0025, 0.0004]])
        self.T = np.array([
            [0, 0.0149, 0.0055],
            [0.0222, 0.7851, 0.0283],
            [0.0053, 0.0348, 0.0150]])
        self.R = np.transpose(self.R)
        self.T = np.transpose(self.T)
        self.RTot = 0.088768
        self.TTot = 0.91123



import context
import unittest

from shorthand import *
from shorthandTest import *
from harmonics import *
from crystal import *
from source import *

class testHarmonicFunctions(unittest.TestCase):
    def testGetXComponents(self):
        testVector1 = complexArray([0.163, 0.5, 0.888]);
        testVector2 = complexArray([0.246, 0.99, 0.2]);
        xComponentsCalculated = getXComponents(testVector1, testVector2);
        xComponentsActual = [0.163, 0.246];
        assertAlmostEqual(xComponentsActual, xComponentsCalculated);

        testVector1 = complexArray([[0.183], [0.5], [0.888]]);
        testVector2 = complexArray([[0.266], [0.99], [0.2]]);
        xComponentsCalculated = getXComponents(testVector1, testVector2);
        xComponentsActual = [0.183, 0.266];
        assertAlmostEqual(xComponentsActual, xComponentsCalculated);

        testVector1 = complexArray([[1.173], [0.7]]);
        testVector2 = complexArray([1.256, 1.99]);
        xComponentsCalculated = getXComponents(testVector1, testVector2);
        xComponentsActual = [1.173, 1.256];
        assertAlmostEqual(xComponentsActual, xComponentsCalculated);

    def testGetYComponents(self):
        testVector1 = complexArray([0.173, 0.4, 0.888]);
        testVector2 = complexArray([0.256, 0.89, 0.2]);
        yComponentsCalculated = getYComponents(testVector1, testVector2);
        yComponentsActual = [0.4, 0.89];
        assertAlmostEqual(yComponentsActual, yComponentsCalculated);

        testVector1 = complexArray([[0.173], [0.5], [0.888]]);
        testVector2 = complexArray([[0.256], [0.99], [0.2]]);
        yComponentsCalculated = getYComponents(testVector1, testVector2);
        yComponentsActual = [0.5, 0.99];
        assertAlmostEqual(yComponentsActual, yComponentsCalculated);

        testVector1 = complexArray([[0.173], [0.7]]);
        testVector2 = complexArray([0.256, 1.99]);
        yComponentsCalculated = getYComponents(testVector1, testVector2);
        yComponentsActual = [0.7, 1.99];
        assertAlmostEqual(yComponentsActual, yComponentsCalculated);

    def testCalculateZeroHarmonicLocation(self):
        harmonicNumber1 = 5;
        harmonicNumber2 = 6;
        numberHarmonics = (harmonicNumber1, harmonicNumber2)
        zeroHarmonicLocationsCalculated = calculateZeroHarmonicLocation(numberHarmonics)
        zeroHarmonicLocationsActual = [2, 3];
        assertAlmostEqual(zeroHarmonicLocationsActual, zeroHarmonicLocationsCalculated);

    def testCalculateMinHarmonic(self):
        harmonicNumber1 = 5;
        harmonicNumber2 = 6;
        numberHarmonics = (harmonicNumber1, harmonicNumber2)
        minHarmonicCalculated= calculateMinHarmonic(numberHarmonics)
        minHarmonicActual = [-2, -3];
        assertAlmostEqual(minHarmonicActual, minHarmonicCalculated);

    def testCalculateMaxHarmonic(self):
        harmonicNumber1 = 5;
        harmonicNumber2 = 6;
        numberHarmonics = (harmonicNumber1, harmonicNumber2)
        maxHarmonicCalculated= calculateMaxHarmonic(numberHarmonics)
        maxHarmonicActual = [2, 2];
        assertAlmostEqual(maxHarmonicActual, maxHarmonicCalculated);

    # ALL THESE TESTS WILL FAIL BECAUSE I NOW NEED TO PASS IN A SOURCE WHICH HAS AN UNKNOWN K0.
    def testGenerateKxMatrix(self):
        absoluteTolerance = 1e-4;
        relativeTolerance = 1e-3;

        # Test our KX matrix at the gamma point
        kxMatrixActual = self.KxMatrixGPoint;
        sourceGPoint = Source()
        sourceGPoint.kIncident = self.incidentKVectorGPoint
        sourceGPoint.k0 = 1
        kxMatrixCalculated = generateKxMatrix(sourceGPoint, self.crystal, self.numberHarmonics)
        assertAlmostEqual(kxMatrixActual, kxMatrixCalculated, absoluteTolerance, relativeTolerance,
                "testHarmonics: Kx at Gamma Point");

        # Test our KX matrix at the X point
        kxMatrixActual = self.KxMatrixXPoint;
        sourceXPoint = Source()
        sourceXPoint.kIncident = self.incidentKVectorXPoint
        sourceXPoint.k0 = 1
        kxMatrixCalculated = generateKxMatrix(sourceXPoint, self.crystal, self.numberHarmonics)
        assertAlmostEqual(kxMatrixActual, kxMatrixCalculated, absoluteTolerance, relativeTolerance,
                "testHarmonics: Kx at X Point");

        # Test our KX matrix at the M point
        kxMatrixActual = self.KxMatrixMPoint;
        sourceMPoint = Source()
        sourceMPoint.kIncident = self.incidentKVectorMPoint
        sourceMPoint.k0 = 1
        kxMatrixCalculated = generateKxMatrix(sourceMPoint, self.crystal, self.numberHarmonics)
        assertAlmostEqual(kxMatrixActual, kxMatrixCalculated, absoluteTolerance, relativeTolerance,
                "testHarmonics: Kx at M Point");

    def testGenerateKyMatrix(self):
        absoluteTolerance = 1e-4;
        relativeTolerance = 1e-3;

        # Test our KY matrix at the gamma point
        kyMatrixActual = self.KyMatrixGPoint;
        sourceGPoint = Source()
        sourceGPoint.kIncident = self.incidentKVectorGPoint
        sourceGPoint.k0 = 1
        kyMatrixCalculated = generateKyMatrix(sourceGPoint, self.crystal, self.numberHarmonics)
        assertAlmostEqual(kyMatrixActual, kyMatrixCalculated, absoluteTolerance, relativeTolerance,
                "testHarmonics: Ky at Gamma Point");

        # Test our KY matrix at the X point
        kyMatrixActual = self.KyMatrixXPoint;
        sourceXPoint = Source()
        sourceXPoint.kIncident = self.incidentKVectorXPoint
        sourceXPoint.k0 = 1
        kyMatrixCalculated = generateKyMatrix(sourceXPoint, self.crystal, self.numberHarmonics)
        assertAlmostEqual(kyMatrixActual, kyMatrixCalculated, absoluteTolerance, relativeTolerance,
                "testHarmonics: Ky at X Point");

        # Test our KY matrix at the M point
        kyMatrixActual = self.KyMatrixMPoint;
        sourceMPoint = Source()
        sourceMPoint.kIncident = self.incidentKVectorMPoint
        sourceMPoint.k0 = 1
        kyMatrixCalculated = generateKyMatrix(sourceMPoint, self.crystal, self.numberHarmonics)
        assertAlmostEqual(kyMatrixActual, kyMatrixCalculated, absoluteTolerance, relativeTolerance,
                "testHarmonics: Ky at M Point");

    def setUp(self):
        self.numberHarmonics = (3, 3, 1)
        self.matrixDimensions = np.prod(self.numberHarmonics);
        self.matrixShape = (self.matrixDimensions, self.matrixDimensions);

        a = 1;
        self.ax = a;
        self.ay = a;
        self.r = 0.35 * a;
        self.er = 9.0;

        self.Nx = 512;
        self.Ny = 512;
        self.dx = self.ax / self.Nx;
        self.dy = self.ay / self.Ny;

        self.t1 = complexArray([self.ax, 0])
        self.t2 = complexArray([0, self.ay])
        self.T1 = (2 * pi / self.ax) * complexArray([1, 0]);
        self.T2 = (2 * pi / self.ay) * complexArray([0, 1]);

        self.incidentKVectorGPoint= 0*self.T1;
        self.incidentKVectorXPoint = 0.5*self.T1;
        self.incidentKVectorMPoint = 0.5*self.T1 + 0.5*self.T2;

        xcoors = np.linspace(-self.ax/2 + self.dx/2, self.ax/2 - self.dx/2, self.Nx);
        ycoors = np.linspace(-self.ay/2 + self.dy/2, self.ay/2 - self.dy/2, self.Ny);
        (X, Y) = np.meshgrid(xcoors, ycoors);
        self.UR = complexOnes((self.Nx, self.Ny));
        self.ER = (self.er-1) * np.heaviside(sq(X) + sq(Y) - sq(self.r),1)
        self.ER = self.ER + 1;
        source = Source()
        self.crystal = Crystal(self.ER, self.UR, self.t1, self.t2)

        # The data for Kx, Ky, and Kz will be re-used at each point of key symmetry
        self.KxMatrixGPoint = complexZeros(self.matrixShape);
        self.KxMatrixGPoint[0,0] = 6.2832;
        self.KxMatrixGPoint[2,2] = -6.2832;
        self.KxMatrixGPoint[3,3] = 6.2832;
        self.KxMatrixGPoint[5,5] = -6.2832;
        self.KxMatrixGPoint[6,6] = 6.2832;
        self.KxMatrixGPoint[8,8] = -6.2832;

        self.KyMatrixGPoint = complexZeros(self.matrixShape);
        self.KyMatrixGPoint[0,0] = 6.2832;
        self.KyMatrixGPoint[1,1] = 6.2832;
        self.KyMatrixGPoint[2,2] = 6.2832;
        self.KyMatrixGPoint[6,6] = -6.2832;
        self.KyMatrixGPoint[7,7] = -6.2832;
        self.KyMatrixGPoint[8,8] = -6.2832;

        self.KxMatrixXPoint = complexZeros(self.matrixShape);
        self.KxMatrixXPoint[0,0] = 9.4248;
        self.KxMatrixXPoint[1,1] = 3.1416;
        self.KxMatrixXPoint[2,2] = -3.1416;
        self.KxMatrixXPoint[3,3] = 9.4248;
        self.KxMatrixXPoint[4,4] = 3.1416;
        self.KxMatrixXPoint[5,5] = -3.1416;
        self.KxMatrixXPoint[6,6] = 9.4248;
        self.KxMatrixXPoint[7,7] = 3.1416;
        self.KxMatrixXPoint[8,8] = -3.1416;

        diagonalValuesKyXPoint = complexArray([6.2832, 6.2832, 6.2832, 0, 0, 0, -6.2832, -6.2832, -6.2832]);
        self.KyMatrixXPoint = np.diag(diagonalValuesKyXPoint);

        diagonalValuesKxMPoint = complexArray([9.4248, 3.1416, -3.1416, 9.4248, 3.1416,
            -3.1416, 9.4248, 3.1416, -3.1416]);
        self.KxMatrixMPoint = np.diag(diagonalValuesKxMPoint);

        diagonalValuesKyMPoint = complexArray([9.4248, 9.4248, 9.4248, 3.1416, 3.1416,
            3.1416,-3.1416, -3.1416, -3.1416]);
        self.KyMatrixMPoint = np.diag(diagonalValuesKyMPoint);

if __name__ == 'main':
    unittest.main()



# DATA FOR THIS FILE TAKEN FROM THE TMM BENCHMARKING DATA PROVIDED BY RUMPF.
import context

import unittest
from shorthandTest import *
from matrices import *
from fresnel import *
from matrixParser import *
from source import Source
from layer import Layer
from solver import *
from results import *
from crystal import Crystal
from plotter import Plotter


class TestRCWASolver1x1(unittest.TestCase):

    def testSetupSource(self):
        kIncidentActual = self.kIncident
        kIncidentCalculated = self.solver.source.kIncident
        assertAlmostEqual(kIncidentActual, kIncidentCalculated,
                self.absoluteTolerance, self.relativeTolerance, "testSolver1x1: kIncident")

    def testSetupKMatrices(self):
        KxActual = self.Kx
        KxCalculated = self.solver.Kx
        assertAlmostEqual(KxActual, KxCalculated,
                self.absoluteTolerance, self.relativeTolerance, "testSolver1x1: Kx")

        KyActual = self.Ky
        KyCalculated = self.solver.Ky
        assertAlmostEqual(KyActual, KyCalculated,
                self.absoluteTolerance, self.relativeTolerance, "testSolver1x1: Ky")

        KzActual = self.KzReflectionRegion
        KzCalculated = self.solver.KzReflectionRegion
        assertAlmostEqual(KzActual, KzCalculated,
                self.absoluteTolerance, self.relativeTolerance, "testSolver1x1: KzReflectionRegion")

        KzActual = self.KzTransmissionRegion
        KzCalculated = self.solver.KzTransmissionRegion
        assertAlmostEqual(KzActual, KzCalculated,
                self.absoluteTolerance, self.relativeTolerance, "testSolver1x1: KzTransmissionRegion")

        KzActual = self.KzGapRegion
        KzCalculated = self.solver.KzGapRegion
        assertAlmostEqual(KzActual, KzCalculated,
                self.absoluteTolerance, self.relativeTolerance, "testSolver1x1: KzGapRegion")

    def testEdgeSMatrices(self):
        self.solver.Solve()
        SActual = self.SReflectionRegion
        SCalculated = self.solver.SReflection
        assertAlmostEqual(SActual, SCalculated,
                self.absoluteTolerance, self.relativeTolerance, "testSolver1x1: SReflection")

        self.solver.Solve()
        SActual = self.STransmissionRegion
        SCalculated = self.solver.STransmission
        assertAlmostEqual(SActual, SCalculated,
                self.absoluteTolerance, self.relativeTolerance, "testSolver1x1: STransmission")

    def testInternalSMatrices(self):
        self.solver.Solve()
        SActual = self.SLayer1
        SCalculated = self.solver.Si[0]
        assertAlmostEqual(SActual, SCalculated,
                self.absoluteTolerance, self.relativeTolerance, "testSolver1x1: Si[0]")

        SActual = self.SLayer2
        SCalculated = self.solver.Si[1]
        assertAlmostEqual(SActual, SCalculated,
                self.absoluteTolerance, self.relativeTolerance, "testSolver1x1: Si[1]")

    def testrtAmplitudeCoefficients(self):
        self.solver.Solve()
        rxActual = self.rx
        ryActual = self.ry
        rzActual = self.rz
        (rxCalculated, ryCalculated, rzCalculated) = (self.solver.rx, self.solver.ry, self.solver.rz)

        assertAlmostEqual(rxActual, rxCalculated,
                self.absoluteTolerance, self.relativeTolerance, "testSolver1x1: rx")
        assertAlmostEqual(ryActual, ryCalculated,
                self.absoluteTolerance, self.relativeTolerance, "testSolver1x1: ry")
        assertAlmostEqual(rzActual, rzCalculated,
                self.absoluteTolerance, self.relativeTolerance, "testSolver1x1: rz")

        txActual = self.tx
        tyActual = self.ty
        tzActual = self.tz
        (txCalculated, tyCalculated, tzCalculated) = (self.solver.tx, self.solver.ty, self.solver.tz)
        assertAlmostEqual(txActual, txCalculated,
                self.absoluteTolerance, self.relativeTolerance, "testSolver1x1: tx")
        assertAlmostEqual(tyActual, tyCalculated,
                self.absoluteTolerance, self.relativeTolerance, "testSolver1x1: ty")
        assertAlmostEqual(tzActual, tzCalculated,
                self.absoluteTolerance, self.relativeTolerance, "testSolver1x1: tz")

        rTEActual = self.rTE
        rTMActual = self.rTM
        rTECalculated = self.solver.rTEM[0]
        rTMCalculated = self.solver.rTEM[1]
        assertAlmostEqual(rTEActual, rTECalculated,
                self.absoluteTolerance, self.relativeTolerance, "testSolver1x1: rTE")
        assertAlmostEqual(rTMActual, rTMCalculated,
                self.absoluteTolerance, self.relativeTolerance, "testSolver1x1: rTM")

    def testEllipsometryData(self):
        self.solver.Solve()
        tanPsiActual = self.tanPsi
        cosDeltaActual = self.cosDelta
        deltaActual = self.delta

        tanPsiCalculated = self.solver.results[0].tanPsi
        deltaCalculated = self.solver.results[0].delta
        cosDeltaCalculated = self.solver.results[0].cosDelta

        assertAlmostEqual(tanPsiActual, tanPsiCalculated,
                self.absoluteTolerance, self.relativeTolerance, "testSolver1x1: tanPsi")
        assertAlmostEqual(deltaActual, deltaCalculated,
                self.absoluteTolerance, self.relativeTolerance, "testSolver1x1: delta")
        assertAlmostEqual(cosDeltaActual, cosDeltaCalculated,
                self.absoluteTolerance, self.relativeTolerance, "testSolver1x1: cosDelta")

    def testRT(self):
        self.solver.Solve()
        RActual = self.R
        TActual = self.T
        (RCalculated, TCalculated) = (self.solver.R, self.solver.T)
        assertAlmostEqual(RActual, RCalculated,
                self.absoluteTolerance, self.relativeTolerance, "testSolver1x1: R")
        assertAlmostEqual(TActual, TCalculated,
                self.absoluteTolerance, self.relativeTolerance, "testSolver1x1: T")

    def testIntegrationMultiWavelength(self):
        testWavelengths = self.solver.source.wavelength*np.arange(0.2,2,0.01)
        self.solver.Solve(testWavelengths)
        #Plotter.plotReflectionSpectra(self.solver.results)

    def setUp(self):
        self.absoluteTolerance = 1e-4
        self.relativeTolerance = 1e-3

        reflectionLayer = Layer(er=1.4, ur=1.2)
        transmissionLayer = Layer(er=1.8, ur=1.6)

        # NOTE: t1 AND t2 MUST BE NORMALIZED BY MULTIPLYING BY k0, OTHERWISE THIS WILL NOT WORK, AS
        # EVERYTHING WAS FORMULATED IN TERMS OF NORMALIZED WAVEVECTORS. I DON'T KNOW OF AN ELEGANT WAY
        # TO DO THIS OTHER THAN REQUIRING A CRYSTAL TO HAVE A SOURCE AS THE INPUT. I DON'T KNOW OF
        # AN EASY WAY TO FIX THIS. I'M GOING TO FUDGE IT FOR NOW.
        wavelength = 2.7
        k0 = 2*pi/wavelength
        theta = 57 * deg
        phi = 23*deg
        pTEM = 1/sqrt(2)*complexArray([1,1j])
        source = Source(wavelength=wavelength, theta=theta, phi=phi, pTEM=pTEM, layer=reflectionLayer)

        thicknessLayer1 = wavelength * 0.25 # should be 0.5
        thicknessLayer2 = wavelength * 0.5 # should be 0.3

        numberHarmonics = (1,1)

        layer1 = Layer(er=2.0, ur=1.0, L=thicknessLayer1)
        layer2 = Layer(er=1.0, ur=3.0, L=thicknessLayer2)
        layerStack = LayerStack(reflectionLayer, layer1, layer2, transmissionLayer)

        self.solver = RCWASolver(layerStack, source, numberHarmonics)

        self.Kx = 1.0006
        self.Ky = 0.4247
        self.KzReflectionRegion = 0.705995
        self.kIncident = complexArray([self.Kx, self.Ky, self.KzReflectionRegion])
        self.KzTransmissionRegion = 1.30324
        self.KzGapRegion = 1

        self.rx = 0.0519 - 0.2856j
        self.ry= -0.4324 + 0.0780j
        self.rz = 0.1866 + 0.3580j
        self.tx = -0.0101 + 0.3577j
        self.ty = 0.4358 - 0.0820j
        self.tz = -0.1343 - 0.2480j
        self.R = 0.4403
        self.T = 0.5597

        self.tanPsi = 1.0538
        self.cosDelta = 0.997733
        self.delta = -0.0673421

        #self.rTE = -0.418308 + 0.183386j
        #self.rTM = -0.222488 - 0.426831j
        self.rTE = -0.591577 + 0.259348j
        self.rTM = -0.60363 + 0.314646j

        self.KzGap = 1
        self.WGap = complexIdentity(2)
        self.VGap = complexArray([
            [0 - 0.4250j, 0 - 1.1804j],
            [0 + 2.0013j, 0 + 0.4250j]])

        self.ALayer1 = complexArray([[2.0049, -0.0427], [-0.0427, 2.0873]]);
        self.BLayer1 = complexArray([[-0.0049, 0.0427], [0.0427, -0.0873]]);
        self.XLayer1 = complexArray([[0.1493 + 0.9888j, 0+0j],[0+0j, 0.1493 + 0.9888j]]);
        self.DLayer1 = complexArray([
            [2.0057 - 0.0003j, -0.0445 + 0.0006j],
            [-0.0445 + 0.0006j, 2.0916 - 0.0013j]]);
        self.ALayer2 = complexArray([[3.8324, 0.2579],[0.2579, 3.3342]]);
        self.BLayer2 = complexArray([[-1.8324, -0.2579], [-0.2579, -1.3342]]);
        self.XLayer2 = complexArray([[-0.4583 - 0.8888j, 0+0j],[0+0j, -0.4583 - 0.8888j]]);
        self.DLayer2 = complexArray([
            [4.3436 - 0.7182j, 0.3604 - 0.1440j],
            [0.3604 - 0.1440j, 3.6475 - 0.4401j]]);
        self.S11Layer1 =  complexArray([
            [0.0039 - 0.0006j, -0.0398 + 0.0060j],
            [-0.0398 + 0.0060j, 0.0808 - 0.0121j]])
        self.S11Layer2 = complexArray([
            [0.6997 - 0.2262j, 0.0517 - 0.0014j],
            [0.0517-0.0014j, 0.5998 - 0.2235j]]);
        self.S12Layer1 = complexArray([
            [0.1490 + 0.9880j, 0.0005 + 0.0017j],
            [0.0005 + 0.0017j, 0.1480 + 0.9848j]]);
        self.S12Layer2 = complexArray([
            [-0.2093 - 0.6406j, 0.0311 + 0.0390j],
            [0.0311 + 0.0390j, -0.2693 - 0.7160j]]);
        self.S21Layer1 = complexArray([
            [0.1490 + 0.9880j, 0.0005 + 0.0017j],
            [0.0005 + 0.0017j, 0.1480 + 0.9848j]]);
        self.S21Layer2= complexArray([
            [-0.2093 - 0.6406j, 0.0311 + 0.0390j],
            [0.0311 + 0.0390j, -0.2693 - 0.7160j]]);
        self.S22Layer1 = complexArray([
            [0.0039 - 0.0006j, -0.0398 + 0.0060j],
            [-0.0398 + 0.0060j, 0.0808 - 0.0121j]]);
        self.S22Layer2 = complexArray([
            [0.6997 - 0.2262j, 0.0517 - 0.0014j],
            [0.0517-0.0014j, 0.5998 - 0.2235j]]);
        self.SLayer1 = complexArray([[self.S11Layer1, self.S12Layer1],[self.S21Layer1, self.S22Layer1]])

        self.SLayer2 = complexArray([[self.S11Layer2, self.S12Layer2],[self.S21Layer2, self.S22Layer2]])

        self.transparentSMatrix = complexZeros((2,2,2,2));
        self.transparentSMatrix[1,0] = complexIdentity(2);
        self.transparentSMatrix[0,1] = complexIdentity(2);

        self.SReflectionRegion = complexZeros((2,2,2,2));
        self.SReflectionRegion[0,0] = complexArray([
            [-0.0800, 0.0761],
            [0.0761, -0.2269]]);
        self.SReflectionRegion[0,1] = complexArray([
            [1.0800, -0.0761],
            [-0.0761, 1.2269]]);
        self.SReflectionRegion[1,0] = complexArray([
            [0.9200, 0.0761],
            [0.0761, 0.7731]]);
        self.SReflectionRegion[1,1] = complexArray([
            [0.0800, -0.0761],
            [-0.0761, 0.2269]]);

        self.STransmissionRegion = complexZeros((2,2,2,2));
        self.STransmissionRegion[0,0] = complexArray([
            [0.2060, 0.0440],
            [0.0440, 0.1209]]);
        self.STransmissionRegion[0,1] = complexArray([
            [0.7940, -0.0440],
            [-0.0440, 0.8791]]);
        self.STransmissionRegion[1,0] = complexArray([
            [1.2060, 0.0440],
            [0.0440, 1.1209]]);
        self.STransmissionRegion[1,1] = complexArray([
            [-0.2060, -0.0440],
            [-0.0440, -0.1209]]);



import context

from crystal import *
from layer import *
from source import *
import unittest
from shorthandTest import *

class Test(unittest.TestCase):
    def testExtractCrystal(self):
        t1 = np.array([1,0,0])
        t2 = np.array([0,1,0])
        testCrystal = Crystal(t1, t2)
        testLayer = Layer(crystal=testCrystal)
        testStack = LayerStack(freeSpaceLayer, testLayer, freeSpaceLayer)

        internalLayerActual = 0
        internalLayerCalculated = testStack.extractCrystalLayer()
        assertEqual(internalLayerActual, internalLayerCalculated)

        testStack = LayerStack(freeSpaceLayer, freeSpaceLayer, testLayer, freeSpaceLayer)

        internalLayerActual = 1
        internalLayerCalculated = testStack.extractCrystalLayer()
        assertEqual(internalLayerActual, internalLayerCalculated)



import sys
sys.path.append('core');
sys.path.append('test')

import unittest
from shorthandTest import *
from matrices import *
from fresnel import *
from convolution import generateConvolutionMatrix
from matrixParser import *

S11Layer2 = numpyArrayFromSeparatedColumnsFile("test/matrixDataOblique/layer2/S11Layer2.txt")
np.savetxt("test/mathematica/S11Layer2.csv",S11Layer2)

S12Layer2 = numpyArrayFromSeparatedColumnsFile("test/matrixDataOblique/layer2/S12Layer2.txt")
np.savetxt("test/mathematica/S12Layer2.csv",S12Layer2)

S21Layer2 = numpyArrayFromSeparatedColumnsFile("test/matrixDataOblique/layer2/S21Layer2.txt")
np.savetxt("test/mathematica/S21Layer2.csv",S21Layer2)

S22Layer2 = numpyArrayFromSeparatedColumnsFile("test/matrixDataOblique/layer2/S22Layer2.txt")
np.savetxt("test/mathematica/S22Layer2.csv",S22Layer2)

S11Layer1 = numpyArrayFromSeparatedColumnsFile("test/matrixDataOblique/layer1/S11Layer1.txt")
np.savetxt("test/mathematica/S11Layer1.csv",S11Layer1)

S12Layer1 = numpyArrayFromSeparatedColumnsFile("test/matrixDataOblique/layer1/S12Layer1.txt")
np.savetxt("test/mathematica/S12Layer1.csv",S12Layer1)

S21Layer1 = numpyArrayFromSeparatedColumnsFile("test/matrixDataOblique/layer1/S21Layer1.txt")
np.savetxt("test/mathematica/S21Layer1.csv",S21Layer1)

S22Layer1 = numpyArrayFromSeparatedColumnsFile("test/matrixDataOblique/layer1/S22Layer1.txt")
np.savetxt("test/mathematica/S22Layer1.csv",S22Layer1)



"""
Module for testing the plane wave source class
"""
import context

from shorthand import *
from layer import Layer
from source import Source
import unittest
from shorthandTest import *


class testSource(unittest.TestCase):
    def testKIncident(self):
        kIncidentActual = complexArray([1.0607, 0.61237, 0.70711])
        kIncidentCalculated = self.source.kIncident
        assertAlmostEqual(kIncidentActual, kIncidentCalculated, self.absoluteTolerance, self.relativeTolerance,
                "kIncident in testSource")

    def setUp(self):
        self.absoluteTolerance = 1e-5
        self.relativeTolerance = 1e-4
        wavelength = 0.02
        theta = 60*deg
        phi = 30*deg
        pTEM = 1 / sqrt(2) * complexArray([1,1j])
        reflectionLayer = Layer(er=2,ur=1)
        self.source = Source(wavelength, theta, phi, pTEM, reflectionLayer)



"""
Run all the core unit tests, not the lengthy and major integration tests
"""
import context
import unittest
import sys

import testCrystal
import testHarmonics
import testLayer
import testMaterial
import testMatrices1x1Harmonics
import testMatrices3x3Harmonics
import testNetlistParser
import testSolver1x1Harmonics
import testSolver3x3Harmonics
import testSource

loader = unittest.TestLoader()
suite = unittest.TestSuite()
suite.addTests(loader.loadTestsFromModule(testCrystal))
suite.addTests(loader.loadTestsFromModule(testHarmonics))
suite.addTests(loader.loadTestsFromModule(testLayer))
suite.addTests(loader.loadTestsFromModule(testMaterial))
suite.addTests(loader.loadTestsFromModule(testSource))
suite.addTests(loader.loadTestsFromModule(testMatrices1x1Harmonics))
suite.addTests(loader.loadTestsFromModule(testMatrices3x3Harmonics))
suite.addTests(loader.loadTestsFromModule(testNetlistParser))
suite.addTests(loader.loadTestsFromModule(testSolver1x1Harmonics))

runner = unittest.TextTestRunner(verbosity=3)
result = runner.run(suite)
numberFailures = len(result.errors)
numberErrors= len(result.failures)
numberIssues = numberFailures + numberErrors

sys.exit(numberIssues)
